# SPDX-License-Identifier: BSD-2-Clause
import os, sys
from pathlib import Path
sys.path.insert(0, os.path.abspath('.'))

from torii_boards import __version__ as boards_version

ROOT_DIR = (Path(__file__).parent).parent


project = 'Torii-HDL Boards'
version = boards_version
release = version.split('+')[0]
copyright = '2022, Shrine Maiden Heavy Industries'
language  = 'en'
docver    = version

extensions = [
	'sphinx.ext.autodoc',
	'sphinx.ext.doctest',
	'sphinx.ext.githubpages',
	'sphinx.ext.intersphinx',
	'sphinx.ext.napoleon',
	'sphinx.ext.todo',
	'sphinxcontrib.platformpicker',
	'myst_parser',
	'sphinx_rtd_theme',
]

with open(".gitignore") as f:
	exclude_patterns = [line.strip() for line in f.readlines()]

source_suffix = {
	'.rst': 'restructuredtext',
	'.md': 'markdown',
}

pygments_style         = 'monokai'
autodoc_member_order   = 'bysource'
graphviz_output_format = 'svg'
todo_include_todos     = True

intersphinx_mapping = {
	'python': ('https://docs.python.org/3', None),
	'torii': ('https://torii.shmdn.link/', None)
}


napoleon_google_docstring = False
napoleon_numpy_docstring  = True
napoleon_use_ivar         = True
napoleon_custom_sections  = [
	'Platform overrides'
]

myst_heading_anchors = 3

templates_path = [
	'_templates',
]


html_context = {
	'display_lower_left': False,
	'current_language'  : language,
	'current_version'   : version,
	'version'           : version,
	'display_github'    : True,
	'github_user'       : 'shrine-maiden-heavy-industries',
	'github_repo'       : 'torii-boards',
	'github_version'    : 'main/docs/',
	'versions'          : [
		('latest', '/latest')
	]
}

html_baseurl     = 'https://torii-boards.shmdn.link/'
html_theme       = 'sphinx_rtd_theme'
html_copy_source = False

html_theme_options = {
	'collapse_navigation' : False,
	'style_external_links': True,
}

html_static_path = [
	'_static'
]

html_css_files = [
	'css/styles.css'
]

html_style = 'css/styles.css'



# SPDX-License-Identifier: BSD-2-Clause

import warnings

from ..altera.de1_soc import DE1SoCPlatform

__all__ = (
	'DE1SoCPlatform',
)

warnings.warn(
	'The `IntelPlatform` and `torii_boards.intel` module have been renamed to `AlteraPlatform` and'
	' `torii_boards.altera` respectively.\n'
	f'Replace all instances of `{__name__}` with `torii_boards.altera.{__name__.split(".")[-1]}`.',
	DeprecationWarning, stacklevel = 2
)


if __name__ == '__main__':
	from ..test.blinky import Blinky
	DE1SoCPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from typing                             import List

from torii.build                        import (
	Connector, Resource, Pins, Clock, Attrs, Subsignal, PinsN
)
from torii.hdl.ir                       import Fragment
from torii.build.run                    import BuildProducts, BuildPlan
from torii.platform.vendor.lattice.ecp5 import ECP5Platform
from torii.platform.resources           import (
	LEDResources, ButtonResources, UARTResource, SDRAMResource
)

__all__ = (
	'Colorlight_5A75B_R70Platform',
)


class Colorlight_5A75B_R70Platform(ECP5Platform):
	device      = 'LFE5U-25F'
	package     = 'BG256'
	speed       = '6'
	default_clk = 'clk25'

	resources   = [
		Resource('clk25', 0, Pins('P6', dir = 'i'), Clock(25e6), Attrs(IO_TYPE = 'LVCMOS33')),

		*LEDResources(
			pins = 'P11', invert = True,
			attrs = Attrs(IO_TYPE = 'LVCMOS33', DRIVE = '4')
		),

		*ButtonResources(
			pins = 'M13', invert = True,
			attrs = Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'UP')
		),

		UARTResource(0,
			tx = 'P11', # led (J19 DATA_LED-)
			rx = 'M13', # btn (J19 KEY+)
			attrs = Attrs(IO_TYPE = 'LVCMOS33')
		),

		# SPIFlash (W25Q32JV)   1x/2x/4x speed
		Resource('spi_flash', 0,
			Subsignal('cs',   PinsN('N8', dir = 'o')),
			# Subsignal('clk', Pins(', dir = 'i')),             # driven through USRMCLK
			Subsignal('cipo', Pins('T8', dir = 'i')),           # Chip: DI/IO0
			Subsignal('copi', Pins('T7', dir = 'o')),           #       DO/IO1
			# Subsignal('wp', PinsN('unknown', dir = 'o')),     #          IO2 Todo
			# Subsignal('hold', PinsN('unknown', dir = 'o')),   #          IO3 Todo
			Attrs(IO_TYPE = 'LVCMOS33')
		),

		# 2x ESMT M12L16161A-5T 1M x 16bit 200MHz SDRAMs (organized as 1M x 32bit)
		# 2x WinBond W9816G6JH-6 1M x 16bit 166MHz SDRAMs (organized as 1M x 32bit) are lso reported
		SDRAMResource(0,
			clk = 'C6', we_n = 'C7', cas_n = 'E7', ras_n = 'D7',
			ba = 'A7', a = 'A9 E10 B12 D13 C12 D11 D10 E9 D9 B7 C8',
			dq = 'B13 C11 C10 A11 C9 E8  B6  B9  A6  B5  A5  B4  B3 C3  A2  B2 '
				'E2  D3  A4  E4  D4 C4  E5  D5  E6  D6  D8  A8  B8 B10 B11 E11',
			attrs = Attrs(PULLMODE = 'NONE', DRIVE = '4', SLEWRATE = 'FAST', IO_TYPE = 'LVCMOS33')
		),

		# Broadcom B50612D Gigabit Ethernet Transceiver
		Resource('eth_rgmii', 0,
			Subsignal('rst',     PinsN('P5', dir = 'o')),
			Subsignal('mdc',     Pins('P3', dir = 'o')),
			Subsignal('mdio',    Pins('T2', dir = 'io')),
			Subsignal('tx_clk',  Pins('M2', dir = 'o')),
			Subsignal('tx_ctl',  Pins('M3', dir = 'o')),
			Subsignal('tx_data', Pins('L1 L3 P2 L4', dir = 'o')),
			Subsignal('rx_clk',  Pins('M1', dir = 'i')),
			Subsignal('rx_ctl',  Pins('N6', dir = 'i')),
			Subsignal('rx_data', Pins('N1 M5 N5 M6', dir = 'i')),
			Attrs(IO_TYPE = 'LVCMOS33')
		),

		# Broadcom B50612D Gigabit Ethernet Transceiver
		Resource('eth_rgmii', 1,
			Subsignal('rst',     PinsN('P5', dir = 'o')),
			Subsignal('mdc',     Pins('P3', dir = 'o')),
			Subsignal('mdio',    Pins('T2', dir = 'io')),
			Subsignal('tx_clk',  Pins('M12', dir = 'o')),
			Subsignal('tx_ctl',  Pins('R15', dir = 'o')),
			Subsignal('tx_data', Pins('T14 R12 R13 R14', dir = 'o')),
			Subsignal('rx_clk',  Pins('M16', dir = 'i')),
			Subsignal('rx_ctl',  Pins('L15', dir = 'i')),
			Subsignal('rx_data', Pins('P13 N13 P14 M15', dir = 'i')),
			Attrs(IO_TYPE = 'LVCMOS33')
		),

	]
	connectors = [
		Connector('j',  1, 'F3  F1  G3  - G2  H3  H5  F15 L2 K1 J5 K2 B16 J14 F12 -'),
		Connector('j',  2, 'J4  K3  G1  - K4  C2  E3  F15 L2 K1 J5 K2 B16 J14 F12 -'),
		Connector('j',  3, 'H4  K5  P1  - R1  L5  F2  F15 L2 K1 J5 K2 B16 J14 F12 -'),
		Connector('j',  4, 'P4  R2  M8  - M9  T6  R6  F15 L2 K1 J5 K2 B16 J14 F12 -'),
		Connector('j',  5, 'M11 N11 P12 - K15 N12 L16 F15 L2 K1 J5 K2 B16 J14 F12 -'),
		Connector('j',  6, 'K16 J15 J16 - J12 H15 G16 F15 L2 K1 J5 K2 B16 J14 F12 -'),
		Connector('j',  7, 'H13 J13 H12 - G14 H14 G15 F15 L2 K1 J5 K2 B16 J14 F12 -'),
		Connector('j',  8, 'A15 F16 A14 - E13 B14 A13 F15 L2 K1 J5 K2 B16 J14 F12 -'),
		Connector('j', 19, ' -  M13  -  - P11'),
	]

	@property
	def required_tools(self) -> List[str]:
		return super().required_tools + [
			'openFPGALoader'
		]

	def toolchain_prepare(self, fragment: Fragment, name: str, **kwargs) -> BuildPlan:
		overrides = dict(ecppack_opts = '--compress')
		overrides.update(kwargs)
		return super().toolchain_prepare(fragment, name, **overrides)

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from subprocess import check_call
		from os         import environ

		tool = environ.get('OPENFPGALOADER', 'openFPGALoader')
		with products.extract(f'{name}.bit') as bitstream_filename:
			check_call([
				tool, '-c', 'ft232', '-m', bitstream_filename
			])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	Colorlight_5A75B_R70Platform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                         import Connector, Resource, Pins, Clock, Attrs
from torii.build.run                     import BuildProducts
from torii.platform.vendor.lattice.ice40 import ICE40Platform
from torii.platform.resources            import (
	LEDResources, UARTResource, SPIFlashResources
)

__all__ = (
	'ICESugarNanoPlatform',
)


class ICESugarNanoPlatform(ICE40Platform):
	device      = 'iCE40LP1K'
	package     = 'CM36'
	default_clk = 'clk12'

	resources   = [
		Resource(
			'clk12', 0, Pins('D1', dir = 'i'), Clock(12e6),
			Attrs(GLOBAL = True, IO_STANDARD = 'LVCMOS33')
		),

		*LEDResources(pins = 'B6', invert = False, attrs = Attrs(IO_STANDARD = 'LVCMOS33')),

		UARTResource(0,
			tx = 'B3', rx = 'A3',
			attrs = Attrs(IO_STANDARD = 'LVTTL33', PULLUP = 1)
		),

		*SPIFlashResources(0,
			cs_n = 'D5', clk = 'E5', copi = 'E4', cipo = 'F5',
			attrs = Attrs(IO_STANDARD = 'LVCMOS33')
		),
	]

	connectors = [
		Connector('pmod', 0, 'E2 D1 B1 A1 - -'),                    # PMOD1
		Connector('pmod', 1, 'B3 A3 B6 C5 - -'),                    # PMOD2
		Connector('pmod', 2, 'B4 B5 E1 B1 - - C6 E3 C2 A1 - -'),    # PMOD3
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		iceprog = environ.get('ICEPROG', 'iceprog')
		with products.extract(f'{name}.bin') as bitstream_filename:
			check_call([iceprog, bitstream_filename])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	ICESugarNanoPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from typing                       import Tuple

from torii.build                  import Resource, Pins, Clock, Attrs, Subsignal, Connector
from torii.build.run              import BuildProducts
from torii.platform.vendor.xilinx import XilinxPlatform
from torii.platform.resources     import LEDResources, DDR3Resource

__all__ = (
	'AlchitryAuPlatform',
)


def find_loader() -> Tuple[str, str]:
	from os     import environ, path
	from shutil import which

	loader_prgm = environ.get('ALCHITRY_LOADER', which('loader'))
	if loader_prgm is None:
		raise EnvironmentError(
			'Could not find Alchrity Loader. Place '
			'it directly in PATH or specify path explicitly via the '
			'ALCHITRY_LOADER environment variable'
		)
	bridge_bin = environ.get('ALCHITRY_BRIDGE_BIN', path.join(path.dirname(loader_prgm), 'au_loader.bin'))
	return (loader_prgm, bridge_bin)


class AlchitryAuPlatform(XilinxPlatform):
	device      = 'XC7A35T' # Artix 7 33K LEs
	package     = 'FTG256'
	speed       = '1'
	default_clk = 'clk100'

	resources   = [
		Resource(
			'clk100', 0, Pins('N14', dir = 'i'), Clock(10e7),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),

		# On-Board LED Array
		*LEDResources(
			pins = 'K13 K12 L14 L13 M16 M14 M12 N16',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')),

		Resource('usb', 0,
			Subsignal('usb_tx', Pins('P16', dir = 'o')),
			Subsignal('usb_rx', Pins('P15', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),

		# TODO: This is untested
		DDR3Resource(0,
			rst_n = 'D13', clk_p = 'G14', clk_n = 'F14', clk_en = 'D15', cs_n = 'D16', we_n = 'E11', ras_n = 'D14', cas_n = 'D14',
			a = 'F12 G16 G15 E16 H11 G12 H16 H12 H16 H13 E12 H14 F13 J15',
			ba = 'E13 F15 E15',
			dqs_p = 'B15 A15', dqs_n = 'B9 A10',
			dq = 'A13 B16 B14 C11 C13 C16 C12 C14 D8 B11 C8 B10 A12 A8 B12 A9',
			dm = 'A14 C9', odt = 'G11',
			diff_attrs = Attrs(IOSTANDARD = 'LVDS'),
			attrs = Attrs(IOSTANDARD = 'LVCMOS15')),
	]

	connectors  = [
		Connector('bank', 0,
			'T8  T7  T5  R5  R8  P8  L2  L3  J1  K1  H1  H2  G1  G2  K5  E6 '
			'T10 T9  R6  R7  P9  N9  K2  K3  J4  J5  H3  J3  H4  H5  N6  M6 '
		),
		Connector('bank', 1,
			'D1  E2  A2  B2  E1  F2  F3  F4  A3  B4  A4  A5  B5  B6  A7  B7 '
			'B1  C1  C2  C3  D3  E3  C4  D4  G4  G5  E5  F5  D5  D6  C6  C7 '
		),
		Connector('bank', 2,
			'T13 R13 T12 R12 R11 R10 N2  N3  P3  P4  M4  L4  N4  M5  L5  P5 '
			'P11 P10 N12 N11 P13 N13 M1  M2  P1  N1  R1  R2  T2  R3  T3  T4 '
		),
		Connector('bank', 3,
			'L14 L13 M12 N16 R16 R15 P14 M15 P16 P15  -   -   -   -   -   - '
			'K13 K12 M16 M14 T16 T14 N14   -   -   -  -   -   -   -   -   - '
		)
	]

	def toolchain_program(self, products: BuildProducts , name: str) -> None:
		from subprocess import check_call

		(loader, bridge_bin) = find_loader()
		with products.extract(f'{name}.bin') as bitstream_filename:
			check_call([
				loader, '-e', '-f', bitstream_filename,
				'-p', bridge_bin
			])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	AlchitryAuPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

try:
	from importlib import metadata
	__version__ = metadata.version(__package__)

except ImportError:
	# No importlib_metadata. This shouldn't normally happen, but some people prefer not installing
	# packages via pip at all, instead using PYTHONPATH directly or copying the package files into
	# `lib/pythonX.Y/site-packages`. Although not a recommended way, we still try to support it.
	__version__ = 'unknown' # :nocov:



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                  import Connector, Resource, Pins, Clock, Attrs
from torii.build.run              import BuildProducts
from torii.platform.vendor.altera import AlteraPlatform
from torii.platform.resources     import LEDResources, ButtonResources, SwitchResources

__all__ = (
	'ArrowDECAPlatform',
)


class ArrowDECAPlatform(AlteraPlatform):
	device      = '10M50DA' # MAX 10
	package     = 'F484'
	speed       = 'C6'
	suffix      = 'GES'
	default_clk = 'clk50'

	resources   = [
		Resource('clk50', 0, Pins('M8',  dir = 'i'), Clock(50e6), Attrs(io_standard = '2.5 V')),
		Resource('clk50', 1, Pins('P11', dir = 'i'), Clock(50e6), Attrs(io_standard = '3.3 V')),
		Resource('clk50', 2, Pins('N15', dir = 'i'), Clock(50e6), Attrs(io_standard = '1.5 V')),
		Resource('clk10', 0, Pins('M9',  dir = 'i'), Clock(10e6), Attrs(io_standard = '2.5 V')),

		*LEDResources(
			pins = 'C7 C8 A6 B7 C4 A5 B4 C5',
			invert = True,
			attrs = Attrs(io_standard = '1.2 V')
		),
		*ButtonResources(
			pins = 'H21 H22',
			invert = True,
			attrs = Attrs(io_standard = '1.5 V')
		),
		*SwitchResources(
			pins = 'J21 J22',
			attrs = Attrs(io_standard = '1.5 V')
		),
	]

	connectors  = [
		Connector('gpio', 0,
			'W18  Y18  Y19  AA17 AA20 AA19 AB21 AB20 AB19 Y16  V16  '
			'AB18 V15  W17  AB17 AA16 AB16 W16  AB15 W15  Y14  AA15 '
			'AB14 AA14 AB13 AA13 AB12 AA12 AB11 AA11 AB10 Y13  Y11  '
			'W13  W12  W11  V12  V11  V13  V14  Y17  W14  U15  R13  '
		),
		Connector('gpio', 1,
			'Y5   Y6   W6   W7   W8   V8   AB8   V7  R11  AB7  AB6  '
			'AA7  AA6  Y7   V10  U7   W9   W5   R9   W4    P9  V17  '
			'W3   -    -    -    -    -    -    -    -     -   -    '
		),
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		quartus_pgm = environ.get('QUARTUS_PGM', 'quartus_pgm')
		with products.extract(f'{name}.sof') as bitstream_filename:
			check_call([
				quartus_pgm, '--haltcc', '--mode', 'JTAG',
				'--operation', 'P;' + bitstream_filename
			])

	@property
	def file_templates(self):
		# Configure the voltages of the I/O banks by appending the global
		# assignments to the template. However, we create our own copy of the
		# file templates before modifying them to avoid modifying the original.
		return {
			**super().file_templates,
			'{{name}}.qsf':
				super().file_templates.get('{{name}}.qsf') +
				r'''
				set_global_assignment -name IOBANK_VCCIO 2.5V -section_id 1A
				set_global_assignment -name IOBANK_VCCIO 2.5V -section_id 1B
				set_global_assignment -name IOBANK_VCCIO 2.5V -section_id 2
				set_global_assignment -name IOBANK_VCCIO 3.3V -section_id 3
				set_global_assignment -name IOBANK_VCCIO 3.3V -section_id 4
				set_global_assignment -name IOBANK_VCCIO 1.5V -section_id 5
				set_global_assignment -name IOBANK_VCCIO 1.5V -section_id 6
				set_global_assignment -name IOBANK_VCCIO 1.8V -section_id 7
				set_global_assignment -name IOBANK_VCCIO 1.2V -section_id 8

				set_global_assignment -name FORCE_CONFIGURATION_VCCIO ON
				set_global_assignment -name AUTO_RESTART_CONFIGURATION OFF
				set_global_assignment -name ENABLE_CONFIGURATION_PINS OFF
				set_global_assignment -name ENABLE_BOOT_SEL_PIN OFF
				'''
		}


if __name__ == '__main__':
	from ..test.blinky import Blinky
	ArrowDECAPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                  import (
	Resource, Pins, Clock, Attrs, Subsignal, Connector, PinsN,
	DiffPairs
)
from torii.build.run              import BuildProducts
from torii.platform.vendor.xilinx import XilinxPlatform
from torii.platform.resources     import (
	SwitchResources, RGBLEDResource, LEDResources, ButtonResources
)

__all__ = (
	'ArtyZ720Platform',
)


class ArtyZ720Platform(XilinxPlatform):
	device      = 'xc7z020'
	package     = 'clg400'
	speed       = '1'
	default_clk = 'clk125'

	resources   = [
		Resource(
			'clk125', 0, Pins('H16', dir = 'i'), Clock(125e6), Attrs(IOSTANDARD = 'LVCMOS33')
		),

		*SwitchResources(
			pins = 'M20 M19',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),

		RGBLEDResource(0,
			r = 'N15', g = 'G17', b = 'L15',                    # LD4
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		RGBLEDResource(1,                                       # LD5
			r = 'M15', g = 'L14', b = 'G14',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),

		*LEDResources(
			pins = 'R14 P14 N16 M14',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),

		*ButtonResources(
			pins = 'D19 D20 L20 L19',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),

		Resource('audio', 0,
			Subsignal('pwm', Pins('R18', dir = 'o')),
			Subsignal('sd',  PinsN('T17', dir = 'o')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),

		Resource('crypto_sda', 0,                               # ATSHA204A
			Pins('J15', dir = 'io'),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),

		Resource('hdmi_rx', 0,                                  # J10
			Subsignal('cec', Pins('H17', dir = 'io')),
			Subsignal('clk', DiffPairs('N18', 'P19', dir = 'i'),
				Attrs(IOSTANDARD = 'TMDS_33')
			),
			Subsignal('d',   DiffPairs('V20 T20 N20', 'W20 U20 P20', dir = 'i'),
				Attrs(IOSTANDARD = 'TMDS_33')
			),
			Subsignal('hpd', Pins('T19', dir = 'o')),
			Subsignal('scl', Pins('U14', dir = 'io')),
			Subsignal('sda', Pins('U15', dir = 'io')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),

		Resource('hdmi_tx', 0,                                  # J11
			Subsignal('cec', Pins('G15', dir = 'io')),
			Subsignal('clk', DiffPairs('L16', 'L17', dir = 'o'),
				Attrs(IOSTANDARD = 'TMDS_33')
			),
			Subsignal('d',   DiffPairs('K17 K19 J18', 'K18 J19 H18', dir = 'o'),
				Attrs(IOSTANDARD = 'TMDS_33')
			),
			Subsignal('hpd', PinsN('R19', dir = 'i')),
			Subsignal('scl', Pins('M17', dir = 'io')),
			Subsignal('sda', Pins('M18', dir = 'io')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		)
	]
	connectors = [
		Connector('pmod', 0, 'Y18 Y19 Y16 Y17 - - U18 U19 W18 W19 - -'),  # JA
		Connector('pmod', 1, 'W14 Y14 T11 T10 - - V16 W16 V12 W13 - -'),  # JB

		Connector('ck_io', 0, {
			# Outer Digital Header
			'io0': 'T14',
			'io1': 'U12',
			'io2': 'U13',
			'io3': 'V13',
			'io4': 'V15',
			'io5': 'T15',
			'io6': 'R16',
			'io7': 'U17',
			'io8': 'V17',
			'io9': 'V18',
			'io10': 'T16',
			'io11': 'R17',
			'io12': 'P18',
			'io13': 'N17',

			# Inner Digital Header
			'io26': 'U5',
			'io27': 'V5',
			'io28': 'V6',
			'io29': 'U7',
			'io30': 'V7',
			'io31': 'U8',
			'io32': 'V8',
			'io33': 'V10',
			'io34': 'W10',
			'io35': 'W6',
			'io36': 'Y6',
			'io37': 'Y7',
			'io38': 'W8',
			'io39': 'Y8',
			'io40': 'W9',
			'io41': 'Y9',

			# Outer Analog Header as Digital IO
			'a0': 'Y11',
			'a1': 'Y12',
			'a2': 'W11',
			'a3': 'V11',
			'a4': 'T5',
			'a5': 'U10',

			# Inner Analog Header as Digital IO
			'a6': 'F19',
			'a7': 'F20',
			'a8': 'C20',
			'a9': 'B20',
			'a10': 'B19',
			'a11': 'A20',

			# Misc.
			'a': 'Y13'
		}),

		Connector('ck_spi', 0, {
			'cipo': 'W15',
			'copi': 'T12',
			'sck': 'H15',
			'ss': 'F16'
		}),

		Connector('ck_i2c', 0, {
			'scl': 'P16',
			'sda': 'P15'
		}),

		Connector('xadc', 0, {
			# Outer Analog Header
			'vaux1_n': 'D18',
			'vaux1_p': 'E17',
			'vaux9_n': 'E19',
			'vaux9_p': 'E18',
			'vaux6_n': 'J14',
			'vaux6_p': 'K14',
			'vaux15_n': 'J16',
			'vaux15_p': 'K16',
			'vaux5_n': 'H20',
			'vaux5_p': 'J20',
			'vaux13_n': 'G20',
			'vaux13_p': 'G19',

			# Inner Analog Header
			'vaux12_n': 'F20',
			'vaux12_p': 'F19',
			'vaux0_n': 'B20',
			'vaux0_p': 'C20',
			'vaux8_n': 'A20',
			'vaux8_p': 'B19'
		})
	]

	def toolchain_program(self, products: BuildProducts, name: str, **kwargs) -> None:
		from os         import environ
		from subprocess import run

		xc3sprog = environ.get('XC3SPROG', 'xc3sprog')
		with products.extract(f'{name}.bit') as bitstream:
			run([xc3sprog, '-c', 'jtaghs1_fast', '-p', '1', bitstream], check = True)


if __name__ == '__main__':
	from ..test.blinky import Blinky
	ArtyZ720Platform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                  import (
	Resource, Pins, Clock, Attrs, Subsignal, Connector, PinsN,
)
from torii.build.run              import BuildProducts
from torii.platform.vendor.xilinx import XilinxPlatform
from torii.platform.resources     import (
	SPIResource, SPIFlashResources, SRAMResource, UARTResource, VGAResource,
	Display7SegResource, PS2Resource,
)


__all__ = (
	'MercuryPlatform',
)


class MercuryPlatform(XilinxPlatform):
	'''
	Original Mercury Board from Micro-Nova: https://www.micro-nova.com

	Mercury Manual: https://www.micro-nova.com/s/mercury_rm.pdf
	Mercury Schematic: https://www.micro-nova.com/s/mercury_schematic.pdf

	The Mercury board is often paired with an extension board called the
	Baseboard, which provides an ample set of I/O for FPGA beginners.

	Baseboard Manual: https://www.micro-nova.com/s/baseboard_rm.pdf
	Baseboard Schematics: https://www.micro-nova.com/s/baseboard_schematic.pdf

	Mercury and Baseboard Resources: https://www.micro-nova.com/resources-mercury
	'''

	device  = 'xc3s200a'
	package = 'vq100'
	speed   = '4'

	default_clk = 'clk50'
	resources = [
		Resource(
			'clk50', 0, Pins('P43', dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS33'), Clock(50e6)
		),

		Resource('button', 0, Pins('P41', dir = 'i'), Attrs(IOSTANDARD = 'LVTTL')),

		# The serial interface and flash memory have a shared SPI bus.
		# FPGA is secondary.
		SPIResource('spi_serial', 0,
			role = 'peripheral',
			cs_n = 'P39', clk = 'P53', copi = 'P46', cipo = 'P51',
			attrs = Attrs(IOSTANDARD = 'LVTTL'),
		),

		# FPGA is primary.
		*SPIFlashResources(0,
			cs_n = 'P27', clk = 'P53', copi = 'P46', cipo = 'P51',
			attrs = Attrs(IOSTANDARD = 'LVTTL')
		),

		# ADC over SPI- FPGA is primary.
		SPIResource('spi_adc', 0,
			role = 'controller',
			cs_n = 'P12', clk = 'P9', copi = 'P10', cipo = 'P21',
			attrs = Attrs(IOSTANDARD = 'LVTTL'),
		),

		# GPIO/SRAM Control
		# 5V tolerant GPIO is shared w/ the SRAM (on 200k gate devices) using
		# this pin. All GPIO except gpio:30 (and gpio:20, though see comment
		# under SRAMResource) interface to the SRAM. On assertion, this signal
		# will tristate the level-shifters, preventing any output on the 5V
		# GPIO pins (including gpio:30 and gpio:20).
		Resource('bussw_oe', 0, PinsN('P30N', dir = 'o'), Attrs(IOSTANDARD = 'LVTTL'))
	]

	# Perhaps define some connectors as having a specific purpose- i.e. a 5V GPIO
	# bus with data, peripheral-select, and control signals?
	connectors = [
		Connector('gpio', 0, '''
			P59 P60 P61 P62 P64 P57
			P56 P52 P50 P49 P85 P84
			P83 P78 P77 P65 P70 P71
			P72 P73 P5  P4  P6  P98
			P94 P93 P90 P89 P88 P86
			'''
		),  # 5V I/O- LVTTL.
		Connector('dio', 0, 'P20 P32 P33 P34 P35 P36 P37'),  # Fast 3.3V IO
		# (Directly attached to FPGA)- LVCMOS33.
		Connector('clkio', 0, 'P40 P44'),  # Clock IO (Can be used as GPIO)-
		# LVCMOS33.
		Connector('input', 0, 'P68 P97 P7 P82'),  # Input-only pins- LVCMOS33.
		Connector('led', 0, 'P13 P15 P16 P19'),  # LEDs can be used as pins
		# as well- LVTTL.
		Connector('pmod', 0, 'P5 P4 P6 P98 P94 P93 P90 P89')  # Baseboard PMOD.
		# Overlaps w/ GPIO bus.
	]

	# Some default useful extensions. Attach to platform using:
	# p.add_resources(p.leds)
	# pmod_btn = plat.request('led')
	leds = [
		Resource('led', 0, Pins('1', dir = 'o', conn = ('led', 0)), Attrs(IOSTANDARD = 'LVTTL')),
		Resource('led', 1, Pins('2', dir = 'o', conn = ('led', 0)), Attrs(IOSTANDARD = 'LVTTL')),
		Resource('led', 2, Pins('3', dir = 'o', conn = ('led', 0)), Attrs(IOSTANDARD = 'LVTTL')),
		Resource('led', 3, Pins('4', dir = 'o', conn = ('led', 0)), Attrs(IOSTANDARD = 'LVTTL')),
	]

	sram = [
		SRAMResource(0,
			cs_n = 'P3', we_n = 'gpio_0:29',
			# According to the schematic, A19/Pin 25 on the SRAM is wired to
			# gpio-0:20. However, according to the SRAM's datasheet, pin 25 is
			# a NC. Do not expose for now.
			a = '''
				gpio_0:1  gpio_0:2  gpio_0:3  gpio_0:4  gpio_0:5  gpio_0:6
				gpio_0:7  gpio_0:8  gpio_0:9  gpio_0:10 gpio_0:11 gpio_0:12
				gpio_0:13 gpio_0:14 gpio_0:15 gpio_0:16 gpio_0:17 gpio_0:18
				gpio_0:19
			''',
			d = '''
				gpio_0:21 gpio_0:22 gpio_0:23 gpio_0:24 gpio_0:25 gpio_0:26
				gpio_0:27 gpio_0:28
			''',
			attrs = Attrs(IOSTANDARD = 'LVTTL', SLEW = 'FAST')
		)
	]

	# The 'serial port' is in fact over SPI. The creators of the board provide
	# a VHDL file for talking over this interface. In light of space
	# constraints and the fact that both the FT245RL and FPGA can BOTH be
	# SPI primaries, however, it may be necessary to sacrifice two 'high-speed'
	# (DIO, INPUT) pins instead.
	serial = [
		# RX: FTDI D0, TX: FTDI D1
		UARTResource(0, rx = 'input_0:1', tx = 'dio_0:1', attrs = Attrs(IOSTANDARD = 'LVCMOS33'))
	]

	# The remaining peripherals only make sense w/ the Baseboard installed.
	# See: http://www.micro-nova.com/mercury-baseboard/
	_switches = [
		Resource('switch', 0, Pins('1', dir = 'i', conn = ('gpio', 0)), Attrs(IOSTANDARD = 'LVTTL')),
		Resource('switch', 1, Pins('2', dir = 'i', conn = ('gpio', 0)), Attrs(IOSTANDARD = 'LVTTL')),
		Resource('switch', 2, Pins('3', dir = 'i', conn = ('gpio', 0)), Attrs(IOSTANDARD = 'LVTTL')),
		Resource('switch', 3, Pins('4', dir = 'i', conn = ('gpio', 0)), Attrs(IOSTANDARD = 'LVTTL')),
		Resource('switch', 4, Pins('5', dir = 'i', conn = ('gpio', 0)), Attrs(IOSTANDARD = 'LVTTL')),
		Resource('switch', 5, Pins('6', dir = 'i', conn = ('gpio', 0)), Attrs(IOSTANDARD = 'LVTTL')),
		Resource('switch', 6, Pins('7', dir = 'i', conn = ('gpio', 0)), Attrs(IOSTANDARD = 'LVTTL')),
		Resource('switch', 7, Pins('8', dir = 'i', conn = ('gpio', 0)), Attrs(IOSTANDARD = 'LVTTL'))
	]

	_buttons = [
		Resource('button', 1, Pins('1', dir = 'i', conn = ('input', 0)), Attrs(IOSTANDARD = 'LVTTL')),
		Resource('button', 2, Pins('2', dir = 'i', conn = ('input', 0)), Attrs(IOSTANDARD = 'LVTTL')),
		Resource('button', 3, Pins('3', dir = 'i', conn = ('input', 0)), Attrs(IOSTANDARD = 'LVTTL')),
		Resource('button', 4, Pins('4', dir = 'i', conn = ('input', 0)), Attrs(IOSTANDARD = 'LVTTL'))
	]

	_vga = [
		VGAResource(0,
			r = 'dio_0:1 dio_0:2 dio_0:3',
			g = 'dio_0:4 dio_0:5 dio_0:6',
			b = 'dio_0:7 clkio_0:1',
			hs = 'led_0:3', vs = 'led_0:4', invert_sync = True,
			attrs = Attrs(IOSTANDARD = 'LVCMOS33', SLEW = 'FAST')
		)
	]

	_extclk = [
		Resource('extclk', 0, Pins('1', dir = 'i', conn = ('clkio', 1)), Attrs(IOSTANDARD = 'LVCMOS33'))
	]

	_sevenseg = [
		Display7SegResource(0,
			a = 'gpio_0:13', b = 'gpio_0:14', c = 'gpio_0:15', d = 'gpio_0:16',
			e = 'gpio_0:17', f = 'gpio_0:18', g = 'gpio_0:19', dp = 'gpio_0:20',
			invert = True, attrs = Attrs(IOSTANDARD = 'LVTTL')
		),
		Resource('display_7seg_ctrl', 0,
			Subsignal('en', Pins('9 10 11 12', dir = 'o', conn = ('gpio', 0))),
			Attrs(IOSTANDARD = 'LVTTL')
		)
	]

	_ps2 = [
		PS2Resource(0,
			clk = '2', dat = '1', conn = ('led', 0), attrs = Attrs(IOSTANDARD = 'LVTTL')
		),
	]

	_audio = [
		Resource('audio', 0,
			Subsignal('l', Pins('30', dir = 'o', conn = ('gpio', 0))),
			Subsignal('r', Pins('29', dir = 'o', conn = ('gpio', 0))),
			Attrs(IOSTANDARD = 'LVTTL')
		)
	]

	baseboard_sram    = _buttons + _vga + _extclk + _ps2
	baseboard_no_sram = baseboard_sram + _switches + _sevenseg + _audio

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		# https://github.com/cr1901/mercpcl
		mercpcl = environ.get('MERCPCL', 'mercpcl')
		with products.extract(f'{name}.bin') as bitstream_filename:
			check_call([mercpcl, bitstream_filename])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	plat = MercuryPlatform()
	plat.add_resources(plat.leds)
	plat.build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                         import Connector, Resource, Pins, Clock, Attrs
from torii.build.run                     import BuildProducts
from torii.platform.vendor.lattice.ice40 import ICE40Platform
from torii.platform.resources            import (
	LEDResources, DirectUSBResource, SPIFlashResources
)

__all__ = (
	'TinyFPGABXPlatform',
)


class TinyFPGABXPlatform(ICE40Platform):
	device      = 'iCE40LP8K'
	package     = 'CM81'
	default_clk = 'clk16'

	resources   = [
		Resource(
			'clk16', 0, Pins('B2', dir = 'i'), Clock(16e6), Attrs(IO_STANDARD = 'SB_LVCMOS')
		),

		*LEDResources(pins = 'B3', attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')),

		DirectUSBResource(0, d_p = 'B4', d_n = 'A4', pullup = 'A3',
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')
		),

		*SPIFlashResources(0,
			cs_n = 'F7', clk = 'G7', copi = 'G6', cipo = 'H7', wp_n = 'H4', hold_n = 'J8',
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')
		),
	]
	connectors  = [
		Connector('gpio', 0,
			# Left side of the board
			#     1  2  3  4  5  6  7  8  9 10 11 12 13
			'   A2 A1 B1 C2 C1 D2 D1 E2 E1 G2 H1 J1 H2 '
			# Right side of the board
			#          14 15 16 17 18 19 20 21 22 23 24
			'         H9 D9 D8 C9 A9 B8 A8 B7 A7 B6 A6 '
			# Bottom of the board
			# 25 26 27 28 29 30 31
			'G1 J3 J4 G9 J9 E8 J2'
		),
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		tinyprog = environ.get('TINYPROG', 'tinyprog')
		with products.extract(f'{name}.bin') as bitstream_filename:
			check_call([tinyprog, '-p', bitstream_filename])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	TinyFPGABXPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                        import (
	Connector, Resource, Pins, Clock, Attrs, PinsN,
	Subsignal, DiffPairs, DiffPairsN
)
from torii.build.run                    import BuildProducts, BuildPlan
from torii.hdl.ir                       import Fragment
from torii.platform.vendor.lattice.ecp5 import ECP5Platform
from torii.platform.resources           import (
	LEDResources, UARTResource, DirectUSBResource, ButtonResources, SDRAMResource
)

__all__ = (
	'Supercon19BadgePlatform',
)


class Supercon19BadgePlatform(ECP5Platform):
	device      = 'LFE5U-45F'
	package     = 'BG381'
	speed       = '8'
	default_clk = 'clk8'

	# The badge's LEDs are wired in a non-straightforward way. Here, the
	# LEDResources represent each of the common anodes of a collection of RGB LEDs.
	# A single one of their cathodes defaults to connected via a FET; the other
	# cathodes are normally-off. Accordingly, they act as normal single-color LEDs
	# unless the cathode signals are explicitly driven.
	#
	# The LEDs on the badge were meant to be a puzzle; so each cathode signal
	# corresponds to a different color on each LED. This means there's no
	# straightforward way of creating pin definitions; you'll need to use the
	# following mapping to find the cathode pin number below.
	led_cathode_mappings = [
		{'r': 0, 'g': 1, 'b': 2}, # LED1: by default, red
		{'r': 2, 'g': 1, 'b': 0}, # LED2: by default, blue
		{'r': 0, 'g': 1, 'b': 2}, # LED3: by default, red
		{'r': 2, 'g': 1, 'b': 0}, # LED4: by default, blue
		{'r': 0, 'g': 1, 'b': 2}, # LED5: by default, red
		{'r': 2, 'g': 1, 'b': 0}, # LED6: by default, blue
		{'r': 2, 'g': 0, 'b': 1}, # LED7: by default, green
		{'r': 0, 'g': 1, 'b': 2}, # LED8: by default, red
		{'r': 0, 'g': 1, 'b': 2}, # LED9: by default, red
		# Note: [LED10 is disconnected by default; bridge R60 to make things work]
		{'r': 2, 'g': 1, 'b': 0}, # LED10: by default, blue
		{'r': 1, 'g': 0, 'b': 2}, # LED11: by default, green
	]

	resources   = [
		Resource('clk8', 0, Pins('U18'), Clock(8e6), Attrs(IO_TYPE = 'LVCMOS33')),

		# Used to trigger FPGA reconfiguration.
		Resource('program', 0, PinsN('R1'), Attrs(IO_TYPE = 'LVCMOS33')),

		# See note above for LED anode/cathode information.
		# Short version is: these work as normal LEDs until you touch their cathodes.
		*LEDResources(
			pins = 'E3 D3 C3 C4 C2 B1 B20 B19 A18 K20 K19',
			attrs = Attrs(IO_TYPE = 'LVCMOS33')
		),
		Resource('led_cathodes', 0, Pins('P19 L18 K18'), Attrs(IO_TYPE = 'LVCMOS33')),

		UARTResource(0, rx = 'U2', tx = 'U1', attrs = Attrs(IO_TYPE = 'LVCMOS33')),

		DirectUSBResource(0,
			d_p = 'F3', d_n = 'G3', pullup = 'E4', vbus_valid = 'F4',
			attrs = Attrs(IO_TYPE = 'LVCMOS33')
		),

		# Buttons, with semantic aliases.
		*ButtonResources(
			pins = 'G2 F2 F1 C1 E1 D2 D1 E2',
			attrs = Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'UP')
		),
		Resource('keypad', 0,
			Subsignal('left',   Pins('G2', dir = 'i')),
			Subsignal('right',  Pins('F2', dir = 'i')),
			Subsignal('up',     Pins('F1', dir = 'i')),
			Subsignal('down',   Pins('C1', dir = 'i')),
			Subsignal('start',  Pins('E1', dir = 'i')),
			Subsignal('select', Pins('D2', dir = 'i')),
			Subsignal('a',      Pins('D1', dir = 'i')),
			Subsignal('b',      Pins('E2', dir = 'i')),
			Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'UP')
		),

		# Direct HDMI on the bottom of the board.
		Resource('hdmi', 0,
			Subsignal('clk', DiffPairsN('P20', 'R20'), Attrs(IO_TYPE = 'TMDS_33')),
			Subsignal('d', DiffPairs('N19 L20 L16', 'N20 M20 L17'), Attrs(IO_TYPE = 'TMDS_33')),
			Subsignal('hpd', PinsN('R18'), Attrs(IO_TYPE = 'LVCMOS33')), # Also called HDMI_HEAC_n
			Subsignal('hdmi_heac_p', PinsN('T19'), Attrs(IO_TYPE = 'LVCMOS33')),
			Attrs(DRIVE = '4'),
		),

		Resource('lcd', 0,
			Subsignal('db',
				Pins('J3 H1 K4 J1 K3 K2 L4 K1 L3 L2 M4 L1 M3 M1 N4 N2 N3 N1'),
			),
			Subsignal('rd',    Pins('P2')),
			Subsignal('wr',    Pins('P4')),
			Subsignal('rs',    Pins('P1')),
			Subsignal('cs',    Pins('P3')),
			Subsignal('id',    Pins('J4')),
			Subsignal('rst',   Pins('H2')),
			Subsignal('fmark', Pins('G1')),
			Subsignal('blen',  Pins('P5')),
			Attrs(IO_TYPE = 'LVCMOS33')
		),

		Resource('spi_flash', 0, # clock needs to be accessed through USRMCLK
			Subsignal('cs',   PinsN('R2')),
			Subsignal('copi', Pins('W2')),
			Subsignal('cipo', Pins('V2')),
			Subsignal('wp',   Pins('Y2')),
			Subsignal('hold', Pins('W1')),
			Attrs(IO_TYPE = 'LVCMOS33')
		),
		Resource('spi_flash_4x', 0, # clock needs to be accessed through USRMCLK
			Subsignal('cs',   PinsN('R2')),
			Subsignal('dq',   Pins('W2 V2 Y2 W1')),
			Attrs(IO_TYPE = 'LVCMOS33')
		),

		# SPI-connected PSRAM.
		Resource('spi_psram_4x', 0,
			Subsignal('cs',  PinsN('D20')),
			Subsignal('clk', Pins('E20')),
			Subsignal('dq',  Pins('E19 D19 C20 F19'), Attrs(PULLMODE = 'UP')),
			Attrs(IO_TYPE = 'LVCMOS33', SLEWRATE = 'SLOW')
		),
		Resource('spi_psram_4x', 1,
			Subsignal('cs',   PinsN('F20')),
			Subsignal('clk',  Pins('J19')),
			Subsignal('dq',   Pins('J20 G19 G20 H20'), Attrs(PULLMODE = 'UP')),
			Attrs(IO_TYPE = 'LVCMOS33', SLEWRATE = 'SLOW')
		),

		SDRAMResource(0, clk = 'D11', cke = 'C11', cs_n = 'C7', we_n = 'B6', ras_n = 'D6', cas_n = 'A6',
			ba = 'A7 C8', a = 'A8 D9 C9 B9 C14 E17 A12 B12 H17 G18 B8 A11 B11',
			dq = 'C5 B5 A5 C6 B10 C10 D10 A9', dqm = 'A10',
			attrs = Attrs(IO_TYPE = 'LVCMOS33', SLEWRATE = 'FAST')
		)
	]

	connectors = [
		Connector('pmod', 0, 'A15 C16 A14 D16 B15 C15 A13 B13'),
		Connector('cartridge', 0,
			'- - - - - - - - C5 B5 A5 C6 B6 A6 D6 C7 A7 C8 B8 A8 D9 C9 B9 A9' # continued:
			' D10 C10 B10 A10 D11 C11 B11 A11 G18 H17 B12 A12 E17 C14'
		),

		# SAO connectors names are compliant with the, erm, SAO 1.69 X-TREME standard.
		# See: https://hackaday.com/2019/03/20/introducing-the-shitty-add-on-v1-69bis-standard/
		Connector('sao', 0, {
			'sda':   'B3', 'scl':   'B2', 'gpio0': 'A2',
			'gpio1': 'A3', 'gpio2': 'B4', 'gpio3': 'A4'
		}),
		Connector('sao', 1, {
			'sda':   'A16', 'scl':   'B17', 'gpio0': 'B18',
			'gpio1': 'A17', 'gpio2': 'B16', 'gpio3': 'C17'
		})
	]

	def toolchain_prepare(self, fragment: Fragment, name: str, **kwargs) -> BuildPlan:
		overrides = dict(ecppack_opts = '--compress --freq 38.8')
		overrides.update(kwargs)
		return super().toolchain_prepare(fragment, name, **overrides, **kwargs)

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		dfu_util = environ.get('DFU_UTIL', 'dfu-util')
		with products.extract(f'{name}.bit') as bitstream_filename:
			check_call([dfu_util, '-d', '1d50:614b', '-a', '0', '-D', bitstream_filename])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	Supercon19BadgePlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                  import (
	Resource, Pins, Clock, Attrs, Subsignal, Connector, PinsN,
	DiffPairs
)
from torii.build.run              import BuildProducts, BuildPlan
from torii.hdl.ir                 import Fragment
from torii.platform.vendor.xilinx import XilinxPlatform
from torii.platform.resources     import (
	SwitchResources, LEDResources, RGBLEDResource, Display7SegResource, VGAResource,
	UARTResource, SDCardResources, PS2Resource, SPIFlashResources
)

__all__ = (
	'Nexys4DDRPlatform',
)


class Nexys4DDRPlatform(XilinxPlatform):
	device      = 'xc7a100t'
	package     = 'csg324'
	speed       = '1'
	default_clk = 'clk100'
	default_rst = 'rst'

	resources   = [
		Resource('clk100', 0, Pins('E3', dir = 'i'), Clock(100e6), Attrs(IOSTANDARD = 'LVCMOS33')),
		Resource('rst', 0, PinsN('C12', dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS33')),

		*SwitchResources(
			pins = {
				0 : 'J15', 1 : 'L16', 2 : 'M13', 3 : 'R15', 4 : 'R17',
				5 : 'T18', 6 : 'U18', 7 : 'R13', 10: 'R16', 11: 'T13',
				12: 'H6',  13: 'U12', 14: 'U11', 15: 'V10'
			},
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		*SwitchResources(
			pins = {8: 'T8', 9: 'U8'},
			attrs = Attrs(IOSTANDARD = 'LVCMOS18')
		),

		*LEDResources(
			pins = 'H17 K15 J13 N14 R18 V17 U17 U16 V16 T15 U14 T16 V15 V14 V12 V11',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),

		RGBLEDResource(0,
			r = 'N15', g = 'M16', b = 'R12',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		RGBLEDResource(1,
			r = 'N16', g = 'R11', b = 'G14',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),

		Display7SegResource(0,
			a = 'T10', b = 'R10', c = 'K16', d = 'K13', e = 'P15',
			f = 'T11', g = 'L18', dp = 'H15', invert = True,
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		Resource('display_7seg_an', 0,
			PinsN('J17 J18 T9 J14 P14 T14 K2 U13', dir = 'o'),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),

		Resource('button_reset', 0, PinsN('C12', dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS33')),
		Resource('button_center', 0, Pins('N17',  dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS33')),
		Resource('button_up', 0, Pins('M18',  dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS33')),
		Resource('button_left', 0, Pins('P17',  dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS33')),
		Resource('button_right', 0, Pins('M17',  dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS33')),
		Resource('button_down', 0, Pins('P18',  dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS33')),

		VGAResource(0,
			r = 'A3 B4 C5 A4',
			g = 'C6 A5 B6 A6',
			b = 'B7 C7 D7 D8',
			hs = 'B11', vs = 'B12',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),

		*SDCardResources(0,
			clk = 'B1',  cmd = 'C1',  cd = 'A1',
			dat0 = 'C2', dat1 = 'E1', dat2 = 'F1', dat3 = 'D2',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		Resource('sd_card_reset', 0,
			Pins('E2', dir = 'o'), Attrs(IOSTANDARD = 'LVCMOS33')
		),

		Resource('accelerometer', 0,                        # ADXL362
			Subsignal('cs',   PinsN('D15',    dir = 'o')),
			Subsignal('clk',  Pins('F15',     dir = 'o')),
			Subsignal('copi', Pins('F14',     dir = 'o')),
			Subsignal('cipo', Pins('E15',     dir = 'i')),
			Subsignal('int',  Pins('B13 C16', dir = 'i'),
				Attrs(IOSTANDARD = 'LVCMOS33', PULLUP = 'TRUE')
			),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),

		Resource('temp_sensor', 0,                          # ADT7420
			Subsignal('scl', Pins('C14', dir = 'o')),
			Subsignal('sda', Pins('C15', dir = 'io')),
			Subsignal('int', Pins('D13', dir = 'i'), Attrs(PULLUP = 'TRUE')),
			Subsignal('ct',  Pins('B14', dir = 'i'), Attrs(PULLUP = 'TRUE')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),

		Resource('microphone', 0,                           # ADMP421
			Subsignal('clk',    Pins('J5', dir = 'o')),
			Subsignal('data',   Pins('H5', dir = 'i')),
			Subsignal('lr_sel', Pins('F5', dir = 'o')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),

		Resource('audio', 0,
			Subsignal('pwm', Pins('A11',  dir = 'o')),
			Subsignal('sd',  PinsN('D12', dir = 'o')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),

		UARTResource(0,
			rx = 'C4', tx = 'D4', rts = 'E5', cts = 'D3',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33'),
			role = 'dce'
		),

		PS2Resource(0,
			clk = 'F4', dat = 'B2',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33', PULLUP = 'TRUE')
		),

		Resource('eth', 0,                                  # LAN8720A
			Subsignal('mdio',   Pins('A9',      dir = 'io')),
			Subsignal('mdc',    Pins('C9',      dir = 'o')),
			Subsignal('reset',  Pins('B3',      dir = 'o')),
			Subsignal('rxd',    Pins('C11 D10', dir = 'io')),
			Subsignal('rxerr',  Pins('C10',     dir = 'io')),
			Subsignal('txd',    Pins('A10 A8',  dir = 'o')),
			Subsignal('txen',   Pins('B9',      dir = 'o')),
			Subsignal('crs_dv', Pins('D9',      dir = 'io')),
			Subsignal('int',    PinsN('B8',     dir = 'io')),
			Subsignal('clk',    Pins('D5',      dir = 'o'), Clock(50e6)),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),

		*SPIFlashResources(0,
			cs_n = 'L13', clk = 'E9', copi = 'K17', cipo = 'K18', wp_n = 'L14', hold_n = 'M14',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),

		Resource('ddr2', 0,                                 # MT47H64M16HR-25:H
			Subsignal('a', Pins('M4 P4 M6 T1 L3 P5 M2 N1 L4 N5 R2 K5 N6 K3', dir = 'o')),
			Subsignal('dq',
				Pins('R7 V6 R8 U7 V7 R6 U6 R5 T5 U3 V5 U4 V4 T4 V1 T3', dir = 'io'),
				Attrs(IN_TERM = 'UNTUNED_SPLIT_50')
			),
			Subsignal('ba',  Pins('P2 P3 R1', dir = 'o')),
			Subsignal('clk', DiffPairs('L6', 'L5', dir = 'o'), Attrs(IOSTANDARD = 'DIFF_SSTL18_I')),
			Subsignal('clk_en', Pins('M1', dir = 'o')),
			Subsignal('cs',  PinsN('K6', dir = 'o')),
			Subsignal('we',  PinsN('N2', dir = 'o')),
			Subsignal('ras', PinsN('N4', dir = 'o')),
			Subsignal('cas', PinsN('L1', dir = 'o')),
			Subsignal('dqs', DiffPairs('U9 U2', 'V9 V2', dir = 'o'), Attrs(IOSTANDARD = 'DIFF_SSTL18_I')),
			Subsignal('dm',  Pins('T6 U1', dir = 'o')),
			Subsignal('odt', Pins('R5',    dir = 'o')),
			Attrs(IOSTANDARD = 'SSTL18_I', SLEW = 'FAST'))
	]
	connectors = [
		Connector('pmod', 0, 'C17 D18 E18 G17 - - D17 E17 F18 G18 - -'),  # JA
		Connector('pmod', 1, 'D14 F16 G16 H14 - - E16 F13 G13 H16 - -'),  # JB
		Connector('pmod', 2, 'K1  F6  J2  G6  - - E7  J3  J4  E6  - -'),  # JC
		Connector('pmod', 3, 'H4  H1  G1  G3  - - H2  G4  G2  F3  - -')   # JD
	]

	def toolchain_prepare(self, fragment: Fragment, name: str, **kwargs) -> BuildPlan:
		overrides = {
			'script_before_bitstream':
				'set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 4 [current_design]',
			'script_after_bitstream':
				'write_cfgmem -force -format bin -interface spix4 -size 16 '
				f'-loadbit \'up 0x0 {name}.bit\' -file {name}.bin',
			'add_constraints':
				'''
				set_property INTERNAL_VREF 0.9 [get_iobanks 34]
				set_property CFGBVS VCCO [current_design]
				set_property CONFIG_VOLTAGE 3.3 [current_design]
				'''
		}
		return super().toolchain_prepare(fragment, name, **overrides, **kwargs)

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import run

		xc3sprog = environ.get('XC3SPROG', 'xc3sprog')
		with products.extract(f'{name}.bit') as bitstream_filename:
			run([xc3sprog, '-c', 'nexys4', bitstream_filename], check = True)


if __name__ == '__main__':
	from ..test.blinky import Blinky
	Nexys4DDRPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

import subprocess

from torii.build                 import Resource, Pins, Clock, Attrs, Subsignal
from torii.platform.vendor.gowin import GowinPlatform
from torii.platform.resources    import (
	RGBLEDResource, ButtonResources, UARTResource, SPIFlashResources
)

__all__ = (
	'TangNanoPlatform',
)

class TangNanoPlatform(GowinPlatform):
	part        = 'GW1N-LV1QN48C6/I5'
	family      = 'GW1N-1'
	default_clk = 'OSC'
	osc_frequency = 24_000_000
	resources   = [
		# This clock is shared with the USB-JTAG MCU and stops when the USB bus is suspended.
		# It probably should not be used, but is included for completeness.
		Resource(
			'clk24', 0, Pins('35', dir = 'i'), Clock(24_000_000), Attrs(IO_TYPE = 'LVCMOS33')
		),

		RGBLEDResource(0, r = '18', g = '16', b = '17', attrs = Attrs(IO_TYPE = 'LVCMOS33')),

		*ButtonResources(pins = '15 14', invert = True, attrs = Attrs(IO_TYPE = 'LVCMOS33')),

		UARTResource(0, tx = '9', rx = '8', attrs = Attrs(IO_TYPE = 'LVCMOS33', PULL_MODE = 'UP')),

		*SPIFlashResources(0,
			cs_n = '19', clk = '20', copi = '22', cipo = '23', wp_n = '24', hold_n = '25',
			attrs = Attrs(IO_TYPE = 'LVCMOS33')
		),

		Resource('lcd', 0,
			Subsignal('clk', Pins('11',                dir = 'o')),
			Subsignal('hs',  Pins('10',                dir = 'o')),
			Subsignal('vs',  Pins('46',                dir = 'o')),
			Subsignal('de',  Pins('5',                 dir = 'o')),
			Subsignal('r',   Pins('27 28 29 30 31',    dir = 'o')),
			Subsignal('g',   Pins('32 33 34 38 39 40', dir = 'o')),
			Subsignal('b',   Pins('41 42 43 44 45',    dir = 'o')),
			Attrs(IO_TYPE = 'LVCMOS33')),

		Resource('lcd_backlight', 0, Pins('47', dir = 'o'), Attrs(IO_TYPE = 'LVCMOS33')),
	]
	connectors  = []

	def toolchain_prepare(self, fragment, name, **kwargs):
		overrides = {
			'add_options':
				'set_option -use_mspi_as_gpio 1 -use_sspi_as_gpio 1',
			'gowin_pack_opts':
				'--sspi_as_gpio --mspi_as_gpio'
		}
		return super().toolchain_prepare(fragment, name, **overrides, **kwargs)

	def toolchain_program(self, products, name):
		with products.extract(f'{name}.fs') as bitstream_filename:
			subprocess.check_call([
				'openFPGALoader', '-b', 'tangnano', bitstream_filename
			])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	TangNanoPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                               import Connector
from torii.platform.vendor.lattice.machxo_2_3l import MachXO2Platform

__all__ = (
	'TinyFPGAAX1Platform',
)


class TinyFPGAAX1Platform(MachXO2Platform):
	device      = 'LCMXO2-256HC'
	package     = 'SG32'
	speed       = '4'

	connectors  = [
		Connector('gpio', 0,
			# Left side of the board
			#  1  2  3  4  5  6  7  8  9 10 11
			'13 14 16 17 20 21 23 25 26 27 28 '
			# Right side of the board
			# 12 13 14 15 16 17 18 19 20 21 22
			'-  -  -  -  4  5  8  9  10 11 12 '
		),
	]
	resources = []
	# This board doesn't have an integrated programmer.



# SPDX-License-Identifier: BSD-2-Clause

from typing                             import List

from torii.build                        import (
	Connector, Resource, Pins, Clock, Attrs, PinsN,
	Subsignal, DiffPairs
)
from torii.build.run                    import BuildProducts, BuildPlan
from torii.hdl.ir                       import Fragment
from torii.platform.vendor.lattice.ecp5 import ECP5Platform
from torii.platform.resources           import (
	RGBLEDResource, SPIFlashResources, SDCardResources, DirectUSBResource
)

__all__ = (
	'OrangeCrabR0_1Platform',
)


class OrangeCrabR0_1Platform(ECP5Platform):
	device      = 'LFE5U-25F'
	package     = 'MG285'
	speed       = '8'
	default_clk = 'clk'

	resources   = [
		Resource('clk', 0, Pins('A9', dir = 'i'), Clock(48e6), Attrs(IO_TYPE = 'LVCMOS33')),

		# Used to reload FPGA configuration.
		# Can enter USB bootloader by assigning button 0 to program.
		Resource('program', 0, PinsN('R16', dir = 'o'), Attrs(IO_TYPE = 'LVCMOS33')),

		RGBLEDResource(0,
			r = 'V17', g = 'T17', b = 'J3', invert = True,
			attrs = Attrs(IO_TYPE = 'LVCMOS33')
		),

		*SPIFlashResources(0,
			cs_n = 'U17', clk = 'U16', cipo = 'T18', copi = 'U18', wp_n = 'R18', hold_n = 'N18',
			attrs = Attrs(IO_TYPE = 'LVCMOS33'),
		),

		Resource('ddr3', 0,
			Subsignal('rst',     PinsN('B1', dir = 'o')),
			Subsignal('clk',     DiffPairs('J18', 'K18', dir = 'o'), Attrs(IO_TYPE = 'SSTL135D_I')),
			Subsignal('clk_en',  Pins('D6', dir = 'o')),
			Subsignal('cs',      PinsN('A12', dir = 'o')),
			Subsignal('we',      PinsN('B12', dir = 'o')),
			Subsignal('ras',     PinsN('C12', dir = 'o')),
			Subsignal('cas',     PinsN('D13', dir = 'o')),
			Subsignal('a',       Pins('A4 D2 C3 C7 D3 D4 D1 B2 C1 A2 A7 C2 C4', dir = 'o')),
			Subsignal('ba',      Pins('B6 B7 A6', dir = 'o')),
			Subsignal('dqs',     DiffPairs('G18 H17', 'B15 A16', dir = 'io'),
				Attrs(IO_TYPE = 'SSTL135D_I', TERMINATION = 'OFF', DIFFRESISTOR = '100')
			),
			Subsignal('dq',      Pins('F17 F16 G15 F15 J16 C18 H16 F18 C17 D15 B17 C16 A15 B13 A17 A13', dir = 'io'),
				Attrs(TERMINATION = '75')
			),
			Subsignal('dm',      Pins('G16 D16', dir = 'o')),
			Subsignal('odt',     Pins('C13', dir = 'o')),
			Attrs(IO_TYPE = 'SSTL135_I', SLEWRATE = 'FAST')
		),

		Resource('ddr3_pseudo_power', 0,
			# pseudo power pins, leave these at their default value
			Subsignal('vcc_virtual', PinsN('A3 B18 C6 C15 D17 D18 K15 K16 K17', dir = 'o')),
			Subsignal('gnd_virtual', Pins('L15 L16 L18', dir = 'o')),
			Attrs(IO_TYPE = 'SSTL135_II', SLEWRATE = 'FAST')
		),

		*SDCardResources(0,
			dat0 = 'J1', dat1 = 'K3', dat2 = 'L3', dat3 = 'M1', clk = 'K1', cmd = 'K2', cd = 'L1',
			attrs = Attrs(IO_TYPE = 'LVCMOS33', SLEWRATE = 'FAST')
		),

		DirectUSBResource(0, d_p = 'N1', d_n = 'M2', pullup = 'N2', attrs = Attrs(IO_TYPE = 'LVCMOS33'))
	]
	connectors = [
		Connector('io', 0, {
			'0':    'N17',
			'1':    'M18',
			'5':    'B10',
			'6':    'B9',
			'9':    'C8',
			'10':   'B8',
			'11':   'A8',
			'12':   'H2',
			'13':   'J2',
			'cipo': 'N15',
			'copi': 'N16',
			'sck':  'R17',
			'scl':  'C9',
			'sda':  'C10'
		}),
		Connector('mcu', 0, {
			'0':    'A10',
			'1':    'C11',
			'2':    'A11',
			'3':    'B11',
		}),
	]

	@property
	def required_tools(self) -> List[str]:
		return super().required_tools + [
			'dfu-suffix'
		]

	@property
	def command_templates(self) -> List[str]:
		return super().command_templates + [
			r'''
			{{invoke_tool('dfu-suffix')}}
				-v 1209 -p 5af0 -a {{name}}.bit
			'''
		]

	def toolchain_prepare(self, fragment: Fragment, name: str, **kwargs) -> BuildPlan:
		overrides = dict(ecppack_opts = '--compress --freq 38.8')
		overrides.update(kwargs)
		return super().toolchain_prepare(fragment, name, **overrides)

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		dfu_util = environ.get('DFU_UTIL', 'dfu-util')
		with products.extract(f'{name}.bit') as bitstream_filename:
			check_call([dfu_util, '-D', bitstream_filename])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	OrangeCrabR0_1Platform().build(Blinky(), do_program = True)
